﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Configuration;
using MySql.Data;
using MySql.Data.MySqlClient;

/// <summary>
/// This class represent a category for image maps
/// </summary>
public class ImageMapCategory
{
    #region Variables

    public Int32 id;
    public string name;

    #endregion

    #region Constructors

    /// <summary>
    /// Create a new category with default properties
    /// </summary>
    public ImageMapCategory()
    {
        // Set values for instance variables
        this.id = 0;
        this.name = "Unknown";

    } // End of the constructor

    /// <summary>
    /// Create a new category from a reader
    /// </summary>
    /// <param name="reader">A reference to a reader</param>
    public ImageMapCategory(MySqlDataReader reader)
    {
        // Set values for instance variables
        this.id = Convert.ToInt32(reader["id"]);
        this.name = reader["name"].ToString();

    } // End of the constructor

    #endregion

    #region Insert methods

    /// <summary>
    /// Add one category
    /// </summary>
    /// <param name="post">A reference to a category</param>
    public static long Add(ImageMapCategory post)
    {
        // Create the long to return
        long idOfInsert = 0;

        // Create the connection and the sql statement
        string connection = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "INSERT INTO image_map_categories (name) VALUES (@name)";

        // The using block is used to call dispose automatically even if there are an exception.
        using (MySqlConnection cn = new MySqlConnection(connection))
        {
            // The Using block is used to call dispose automatically even if there are an exception.
            using (MySqlCommand cmd = new MySqlCommand(sql, cn))
            {

                // Add parameters
                cmd.Parameters.AddWithValue("@name", post.name);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to
                // avoid having our application crash in such cases
                try
                {
                    // Open the connection
                    cn.Open();

                    // Execute the insert
                    cmd.ExecuteNonQuery();
                    idOfInsert = cmd.LastInsertedId;

                }
                catch (Exception e)
                {
                    string exceptionMessage = "";
                    if (HttpContext.Current.IsDebuggingEnabled)
                        throw e;
                    else
                        exceptionMessage = e.Message;
                }
            }
        }

        // Return the id of the inserted item
        return idOfInsert;

    } // End of the Add method

    #endregion

    #region Update methods

    /// <summary>
    /// Update a category
    /// </summary>
    /// <param name="post">A reference to a category</param>
    public static void Update(ImageMapCategory post)
    {
        // Create the connection and the sql statement
        string connection = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "UPDATE image_map_categories SET name = @name WHERE id = @id";

        // The using block is used to call dispose automatically even if there are an exception.
        using (MySqlConnection cn = new MySqlConnection(connection))
        {
            // The Using block is used to call dispose automatically even if there are an exception.
            using (MySqlCommand cmd = new MySqlCommand(sql, cn))
            {
                // Add parameters
                cmd.Parameters.AddWithValue("@id", post.id);
                cmd.Parameters.AddWithValue("@name", post.name);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to
                // avoid having our application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the update
                    cmd.ExecuteNonQuery();

                }
                catch (Exception e)
                {
                    string exceptionMessage = "";
                    if (HttpContext.Current.IsDebuggingEnabled)
                        throw e;
                    else
                        exceptionMessage = e.Message;
                }
            }
        }

    } // End of the Update method

    #endregion

    #region Get methods

    /// <summary>
    /// Get a list of all categories
    /// </summary>
    /// <returns>A list of category posts</returns>
    public static List<ImageMapCategory> GetAll()
    {
        // Create the list to return
        List<ImageMapCategory> posts = new List<ImageMapCategory>();

        // Create the connection string and the sql statement. Limit with two arguments, 
        // the first argument specifies the offset of the first row to return, and the second specifies the 
        // maximum number of rows to return. The offset of the initial row is 0 (not 1).
        string connection = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "SELECT * FROM image_map_categories ORDER BY id ASC";

        // The using block is used to call dispose automatically even if there are an exception.
        using (MySqlConnection cn = new MySqlConnection(connection))
        {
            // The using block is used to call dispose automatically even if there is a exception.
            using (MySqlCommand cmd = new MySqlCommand(sql, cn))
            {
                // Create a MySqlDataReader
                MySqlDataReader reader = null;

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to
                // avoid having our application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Fill the reader with data from the select command.
                    reader = cmd.ExecuteReader();

                    // Loop through the reader as long as there is something to read.
                    while (reader.Read())
                    {
                        posts.Add(new ImageMapCategory(reader));
                    }
                }
                catch (Exception e)
                {
                    string exceptionMessage = "";
                    if (HttpContext.Current.IsDebuggingEnabled)
                        throw e;
                    else
                        exceptionMessage = e.Message;
                }
                finally
                {
                    // Call Close when done reading to avoid memory leakage.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        // Return the list of posts
        return posts;

    } // End of the GetAll method

    /// <summary>
    /// Get one category on id
    /// </summary>
    /// <param name="id">A id</param>
    /// <returns>A reference to a category post</returns>
    public static ImageMapCategory GetOneById(Int32 id)
    {
        // Create the post to return
        ImageMapCategory post = null;

        // Create the connection and the sql statement
        string connection = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "SELECT * FROM image_map_categories WHERE id = @id LIMIT 1";

        // The using block is used to call dispose automatically even if there are an exception.
        using (MySqlConnection cn = new MySqlConnection(connection))
        {
            // The Using block is used to call dispose automatically even if there are an exception.
            using (MySqlCommand cmd = new MySqlCommand(sql, cn))
            {
                // Add a parameters
                cmd.Parameters.AddWithValue("@id", id);

                // Create a MySqlDataReader
                MySqlDataReader reader = null;

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to
                // avoid having our application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Fill the reader with one row of data.
                    reader = cmd.ExecuteReader();

                    // Loop through the reader as long as there is something to read and add values
                    while (reader.Read())
                    {
                        post = new ImageMapCategory(reader);
                    }
                }
                catch (Exception e)
                {
                    string exceptionMessage = "";
                    if (HttpContext.Current.IsDebuggingEnabled)
                        throw e;
                    else
                        exceptionMessage = e.Message;
                }
                finally
                {
                    // Call Close when done reading to avoid memory leakage.
                    if (reader != null)
                        reader.Close();
                }
            }
        }

        // Return the post
        return post;

    } // End of the GetOneById method

    #endregion

    #region Delete methods
    
    /// <summary>
    /// Delete a category on id
    /// </summary>
    /// <param name="id">The id number for the category</param>
    public static void DeleteOnId(int id)
    {
        // Create the connection and the sql statement
        string connection = ConfigurationManager.ConnectionStrings["ConnectionString"].ToString();
        string sql = "DELETE FROM image_map_categories WHERE id = @id";

        // The using block is used to call dispose automatically even if there are an exception.
        using (MySqlConnection cn = new MySqlConnection(connection))
        {
            // The Using block is used to call dispose automatically even if there are an exception.
            using (MySqlCommand cmd = new MySqlCommand(sql, cn))
            {
                // Add parameters
                cmd.Parameters.AddWithValue("@id", id);

                // The Try/Catch/Finally statement is used to handle unusual exceptions in the code to
                // avoid having our application crash in such cases.
                try
                {
                    // Open the connection.
                    cn.Open();

                    // Execute the update
                    cmd.ExecuteNonQuery();

                }
                catch (Exception e)
                {
                    string exceptionMessage = "";
                    if (HttpContext.Current.IsDebuggingEnabled)
                        throw e;
                    else
                        exceptionMessage = e.Message;
                }
            }
        }

    } // End of the DeleteOnId method

    #endregion

} // End of the class